/*
   $Id: CommandStat.cpp 648 2011-10-31 12:59:22Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

#include "utils.h"
#include "ParametersStat.h"
#include "Bank.h"
#include "masque.h"
#include "GlintEngine.h"
#include "CompactTree.h"
#include "scoring.h"
#include "bitcount.h"
#include "alignment.h"

#include "ParametersStat.h"
#include "CommandStat.h"

void Usage();
//extern STrivialLogger* logger;

#ifndef __LP64__
    #error "GLINT MAY BE COMPILED ONLY ON A 64 BITS PLATFORM"
#endif

/*////////////// Main //////////////*/
int CommandStat::Main(int argc,char* argv[])
{
	try
	{
		Parameters& parameters = Parameters::Instance();
		ParametersInitializer& prm_init = _GetInitializer();
		prm_init.Init(argc, argv, *this);

		Mask::Init();
		Bank::Init();
		BitCount::Init();
		ScoreMatrix::Init();
		MatchConstants::Init();
		CompactTree::Init();

		if ( !parameters.IsSilent() ) 
			PrintHeader();
		
		/* Nothing logged by this program (using STrivialLogger) */
//	    auto_ptr<STrivialLogger> logger_ptr = auto_ptr<STrivialLogger> (new STrivialLogger("file","level"));
//	    logger = logger_ptr.get();
			
		CompactTree*  ctree_ptr   = NULL;
 		Mask& mask                = Mask::Instance();
 		MatchConstants& match_cts = MatchConstants::Instance();

 		/* Displaying the mask */
		for (int i=ROOT_LEVEL;i<=mask.GetLeafLevel();i++)
		{
			cout << "Hit value at level " << i << " : " << match_cts.score_matrix[i] << "\n";
		}

		/* Displaying the indexation tree */
		if (parameters.IsStatPositions() || parameters.IsStatFrequency())
		{
			if (parameters.GetStatLevel() > mask.GetLeafLevel())
			{
				ostringstream msg;
				msg << "ERROR - --stat-level should be in the range ";
				msg << "[";
				msg << ROOT_LEVEL;
				msg << '-';
				msg << mask.GetLeafLevel();
				msg << ']';
				throw(runtime_error(msg.str()));
			}

		    // The bank is not indexed - NE MARCHE PAS ON VERRA PLUS TARD - MANU
		    if ( parameters.IsIndexMode() )
		    {
				throw(runtime_error("ERROR - INDEXING IN THE FLY WITH glint stat NOT YET IMPLEMENTED. Call glint index FIRST\n"));
				/*
			    Bank* bank_ptr            = Bank::InstancePtr();

				// We want to save the index
				if ( parameters.IsSaveIndexMode() )
				{
				    bank_ptr -> Index(false);
				    bank_ptr -> Remove();
					parameters.InitializeFromCommandLine(argc,argv,Usage,GLINTSTAT);
					cerr << (parameters.IsIndexMode() ? " INDEX MODE\n" : " PAS INDEX MODE\n");
				    bank_ptr = Bank::InstancePtr();
				}
				
				// No use saving the index (default): just keep them in memory
				else
				{
					bank_ptr -> Index(true);
				}
	 
			 	ctree_ptr -> Remove();
			 	*/
			}
		 	ctree_ptr        = CompactTree::InstancePtr();
			cout << *ctree_ptr << '\n';
		}
	}
   	catch ( exception& e)
	{	
		cout << e.what() << endl;
		return 1;
	};
	
	return 0;
	
}

void CommandStat::Usage() const
{
	PrintHeader();
	cerr << "glint stat: Display some stat about the indexed bank - Mainly useful for developers\n\n";
	cerr << "Usage:\tglint stat [--frequency][--descending][--cutoff <0-255>][--level <";
	cerr << ROOT_LEVEL << "-leaf level>][--positions][--all-positions] bank.fasta \n";
	cerr <<	"\tbank.fasta: this fasta or multifasta file has been indexed by glintindex or by glint --save-index\n";
	cerr << "\t\t --frequency  : Print the indexed words with their frequency, most frequent first\n";
	cerr << "\t\t --descending : with --frequency, print the indexed words less frequent first\n";
	cerr << "\t\t --cutoff:      with --frequency, do not print an indexed word is number of positions lower than stat-cutoff (def 0)\n";
	cerr << "\t\t                The stat-cutoff switch may be useful to save memory !!!\n";
	cerr << "\t\t --level :      With --frequency, the level in the tree at which the stats are done (def leaf level)\n";
	cerr << "\t\t --positions  : Print the tree, the words, and the (incomplete) positions list in a nice formatted way\n";
	cerr << "\t\t --all-positions : Same as --positions, but print ALL positions (the default is to print only the first 10 positions)\n";
}
