/*
   $Id: SeqInfo.h 793 2014-08-21 07:32:12Z tfaraut $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

#ifndef __SEQINFO_H__
#define __SEQINFO_H__

//#include <cstdint>
#include "stdint.h"
#include <iostream>
#include <string>
#include <vector>
using namespace std;

#include "ptypes.h"
#include "SeqInfo.h"

/**
 * \brief The struct SequencePosition stores in a single object the start, end position and the name of a sequence
 *        start, end positions are taken relative to the buffer in memory (0-based)
 */

struct SequencePosition
{
    SequencePosition(int64_t s=0, int64_t e=0, string n=""):start(s),end(e),Name(n){};
    int64_t start;
    int64_t end;
    string Name;
    uint64_t Size() const { return  (end - start + 1); };
};

ostream & operator<<(ostream&, SequencePosition const &);
istream & operator>>(istream& is, SequencePosition&);

/**
 * \brief The class SeqInfo is a wrapper around a vector of SequencePositions
 */
class SeqInfo
{
	public:
		SeqInfo (const string& s="", CompressionAlgorithm compression=NOCOMPRESSION);
		void push_back(const SequencePosition& sp) { sequence_positions.push_back(sp); };
		SequencePosition operator[](size_t i) const { return sequence_positions[i]; };
		uint32_t GetLength(size_t i) const { return sequence_positions[i].end - sequence_positions[i].start + 1; };
		vector<SequencePosition>::const_iterator begin() const { return sequence_positions.begin(); };
		vector<SequencePosition>::const_iterator end() const { return sequence_positions.end(); };
		size_t size() const { return sequence_positions.size();};
		bool IsOldIndex() { return __old_index; };
		
	private:
		vector<SequencePosition> sequence_positions;
		bool __old_index;
};

/**
 * \brief The class ReverseComplement is a functor used to reverse complement a sequence or a base
 */

class ReverseComplement: public Singleton<ReverseComplement>
{	
	public:	
		friend class Singleton<ReverseComplement>;
		char operator[](uint8_t i) { return __rev[i]; };
		void operator()(unsigned char*, uint32_t);
		string operator()(const string&);

	private:
		ReverseComplement();
		char __rev[256];
};

/**
 * \brief The function Reverse is used to reverse a string (a quality-string, for example)
 */
void Reverse(unsigned char*, uint32_t);
string Reverse(const string&);

/* This is a hack, necessary because of a bug in gcc - cf. Buffer::Reverse */
inline void Reverse1(unsigned char*c, uint32_t l) { Reverse(c,l);}

#endif
