/*
   $Id: bitcount.cpp 649 2011-10-31 13:57:42Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/
#include <stdlib.h>
#include <stdio.h>
#include <limits.h>
#include "bitcount.h"

/* The constructor: initializes the array bits_in_16bits, which will be used by GetbitCounts */
BitCount::BitCount()
{	
	unsigned int i ;
    for (i = 0; i < (0x1u<<16); i++)
        bits_in_16bits [i] = __iterated_bitcount (i) ;
    return ;
}

/* Iterated bitcount iterates over each bit. The while condition sometimes helps
   terminates the loop earlier */
int BitCount::__iterated_bitcount (unsigned int n)
{
    int count=0;
    while (n)
    {
        count += n & 0x1u ;
        n >>= 1 ;
    }
    return count ;
}

/* was: precomputed16_bitcount */
int BitCount::GetBitCount( uint64_t n ) const
{
    // works only for 64-bit int
    return bits_in_16bits [n         & 0xffffu]
        +  bits_in_16bits [(n >> 16) & 0xffffu]
		  +  bits_in_16bits [(n >> 32) & 0xffffu]
		  +  bits_in_16bits [(n >> 48) & 0xffffu];
}
